﻿//#define LAYOUTMODE //uncomment to use designer to edit form vs express

/* Dialog for Channel Mix Effect
 *
 * Programmer: Matthew Reynolds
 * 
 * v 1.0 - April 10 2009
 * 
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace Matthew.ChannelMixEffect
{
 
#if LAYOUTMODE
        public partial class ChannelMixDialog :  Form
#else
    public partial class ChannelMixDialog : PaintDotNet.Effects.EffectConfigDialog<ChannelMixEffect, ChannelMixToken>
#endif
        
    {
        static int TRACKBARMAX = 300;
        Dictionary<Object, Label> trackBarLabelPair; //trackbar, label pair.  used to set label text corresponding to trackbar value
        ComboBox []processOrder; //order of rendering channels, the selected index is the channel


        public ChannelMixDialog()
        { 
            InitializeComponent();
            tbScaleA.Maximum = tbScaleB.Maximum = tbScaleG.Maximum = tbScaleR.Maximum = TRACKBARMAX;


            processOrder = new ComboBox[] { cbFirst, cbSecond, cbThird, cbForth };

            cbSrcA.DataSource = Enum.GetValues(typeof(SourceEnum)); //sources to set channel to
            cbSrcR.DataSource = Enum.GetValues(typeof(SourceEnum));
            cbSrcG.DataSource = Enum.GetValues(typeof(SourceEnum));
            cbSrcB.DataSource = Enum.GetValues(typeof(SourceEnum));

            cbFirst.DataSource = Enum.GetValues(typeof(ChannelsEnum));  //a,r,g,b
            cbSecond.DataSource = Enum.GetValues(typeof(ChannelsEnum));
            cbThird.DataSource = Enum.GetValues(typeof(ChannelsEnum));
            cbForth.DataSource = Enum.GetValues(typeof(ChannelsEnum));
                      
            trackBarLabelPair = new Dictionary<Object, Label>() 
            {
                {tbSetA, lblSetA }, {tbSetR, lblSetR },{tbSetG, lblSetG },{tbSetB, lblSetB },
                {tbAddA, lblAddA }, {tbAddR, lblAddR },{tbAddG, lblAddG },{tbAddB, lblAddB },
                {tbScaleA, lblScaleA }, {tbScaleR, lblScaleR },{tbScaleG, lblScaleG },{tbScaleB, lblScaleB },
            };

            xbEnAddA.Checked = xbEnAddA.Checked = xbEnAddA.Checked = xbEnAddA.Checked = true;

            InitDialogFromToken(initToken);

            xbEnScaleA.Checked = xbEnScaleR.Checked = xbEnScaleG.Checked = xbEnScaleB.Checked = false;
            xbEnA.Checked = xbEnR.Checked = xbEnG.Checked = xbEnB.Checked = false;
            xbEnAddA.Checked = xbEnAddA.Checked = xbEnAddA.Checked = xbEnAddA.Checked = false;           
        }

        ChannelMixToken initToken;
        protected override ChannelMixToken CreateInitialToken()
        {
            initToken = new ChannelMixToken();
            return initToken;
        }

        //set controls from ChannelOptions
        void SetControlSet(
            ChannelOptions copt,
            CheckBox en,
            ComboBox src,
            TrackBar set,
            TrackBar add,
            TrackBar scale,
            CheckBox invert,
            CheckBox enAdd,
            CheckBox enScale)
        {
            en.Checked = copt.enable;
            src.SelectedIndex = (int)copt.source;
            invert.Checked = copt.invert;
            enAdd.Checked = copt.addEn;
            enScale.Checked = copt.scaleEn;

            set.Value = copt.set;
            trackBarLabelPair[set].Text = string.Format("set ({0})", copt.set);
            add.Value = copt.add;
            trackBarLabelPair[add].Text = string.Format("add ({0})", copt.add);
            scale.Value = (int)(copt.scale / ChannelOptions.MAXSCALE * TRACKBARMAX);
            trackBarLabelPair[scale].Text = string.Format("scale ({0})", copt.scale);

        }
        //set ChannelOptions from controls
        void LoadControlSet(
           ChannelOptions copt,
           CheckBox en,
           ComboBox src,
           TrackBar set,
           TrackBar add,
           TrackBar scale,
           CheckBox invert,
           CheckBox enAdd,
           CheckBox enScale)
        {
            copt.enable = en.Checked;
            if (copt.enable)
            {
                copt.source = (SourceEnum)src.SelectedIndex;
                copt.set = set.Value;
                copt.add = add.Value;
                copt.scale = scale.Value * ChannelOptions.MAXSCALE / TRACKBARMAX;
                copt.invert = invert.Checked;
                copt.addEn = enAdd.Checked;
                copt.scaleEn = enScale.Checked;
            }
        }

        protected override void InitDialogFromToken(ChannelMixToken token)
        { //set dialog from token
             SetControlSet(token.A,
                xbEnA,
                cbSrcA,
                tbSetA, tbAddA, tbScaleA,
                xbInvA, xbEnAddA, xbEnScaleA);
             SetControlSet(token.R,
                 xbEnR,
                 cbSrcR,
                 tbSetR, tbAddR, tbScaleR,
                 xbInvR, xbEnAddR, xbEnScaleR);
             SetControlSet(token.G,
                 xbEnG,
                 cbSrcG,
                 tbSetG, tbAddG, tbScaleG,
                 xbInvG, xbEnAddG, xbEnScaleG);
             SetControlSet(token.B,
                 xbEnB,
                 cbSrcB,
                 tbSetB, tbAddB, tbScaleB,
                 xbInvB, xbEnAddB, xbEnScaleB);

             updatingOrder = true;
             cbFirst.SelectedIndex = (int)token.processOrder[0].channel;
             cbSecond.SelectedIndex = (int)token.processOrder[1].channel;
             cbThird.SelectedIndex = (int)token.processOrder[2].channel;
             cbForth.SelectedIndex = (int)token.processOrder[3].channel;
             updatingOrder = false;
        }
        protected override void LoadIntoTokenFromDialog(ChannelMixToken token)
        { //set token from dialog
            LoadControlSet(token.A,
                xbEnA,
                cbSrcA,
                tbSetA, tbAddA, tbScaleA,
                xbInvA, xbEnAddA, xbEnScaleA);
            LoadControlSet(token.R,
                xbEnR,
                cbSrcR,
                tbSetR, tbAddR, tbScaleR,
                xbInvR, xbEnAddR, xbEnScaleR);
            LoadControlSet(token.G,
                xbEnG,
                cbSrcG,
                tbSetG, tbAddG, tbScaleG,
                xbInvG, xbEnAddG, xbEnScaleG);
            LoadControlSet(token.B,
                xbEnB,
                cbSrcB,
                tbSetB, tbAddB, tbScaleB,
                xbInvB, xbEnAddB, xbEnScaleB);

            for (int n = 0; n < 4; n++)
            {
                switch ((ChannelsEnum)processOrder[n].SelectedIndex)
                {//selected index is channel
                    case ChannelsEnum.Alpha: token.processOrder[n] = token.A; break;
                    case ChannelsEnum.Red: token.processOrder[n] = token.R; break;
                    case ChannelsEnum.Green: token.processOrder[n] = token.G; break;
                    case ChannelsEnum.Blue: token.processOrder[n] = token.B; break;
                }
            }
        }

        private void SET_Change(object sender, EventArgs e)
        {
            trackBarLabelPair[sender].Text = string.Format("set ({0})", ((TrackBar)sender).Value);
            FinishTokenUpdate();
        }

        private void ADD_Change(object sender, EventArgs e)
        {
            if (sender == tbAddA) { xbEnAddA.Checked = true; xbEnA.Checked = true; }
            else if (sender == tbAddR) { xbEnAddR.Checked = true; xbEnR.Checked = true; }
            else if (sender == tbAddG) { xbEnAddG.Checked = true; xbEnG.Checked = true; }
            else if (sender == tbAddB) { xbEnAddB.Checked = true; xbEnB.Checked = true; }

            trackBarLabelPair[sender].Text = string.Format("add ({0})", ((TrackBar)sender).Value);
            

            FinishTokenUpdate();
        }

        private void SCALE_Change(object sender, EventArgs e)
        {

            if (sender == tbScaleA) { xbEnScaleA.Checked = true; xbEnA.Checked = true; }
            else if (sender == tbScaleR) { xbEnScaleR.Checked = true; xbEnR.Checked = true; }
            else if (sender == tbScaleG) { xbEnScaleG.Checked = true; xbEnG.Checked = true; }
            else if (sender == tbScaleB) { xbEnScaleB.Checked = true; xbEnB.Checked = true; }

            trackBarLabelPair[sender].Text = string.Format("scale ({0:f2})", ((TrackBar)sender).Value * ChannelOptions.MAXSCALE / TRACKBARMAX);

            FinishTokenUpdate();
        }

        private void Enabled_Click(object sender, EventArgs e)
        { //all enabled checkboxes
            FinishTokenUpdate();
        }

        private void Src_Changed(object sender, EventArgs e)
        { //all source combo boxes
            FinishTokenUpdate();
        }

        private void Button_Click(object sender, EventArgs e)
        {
            if (sender == btnOk)
            {
                FinishTokenUpdate();
                DialogResult = DialogResult.OK;
                Close();
            }
            if (sender == btnCancel)
            {
                DialogResult = DialogResult.Cancel;
                Close();
            }
        }


        private void RESET_Click(object sender, EventArgs e)
        {
            if (sender == btnReset_A_SET || sender == btnResetA || sender == btnReset) tbSetA.Value = 0; //set
            if (sender == btnReset_R_SET || sender == btnResetR || sender == btnReset) tbSetR.Value = 0;
            if (sender == btnReset_G_SET || sender == btnResetG || sender == btnReset) tbSetG.Value = 0;
            if (sender == btnReset_B_SET || sender == btnResetB || sender == btnReset) tbSetB.Value = 0;
            if (sender == btnReset_A_ADD || sender == btnResetA || sender == btnReset) tbAddA.Value = 0; //add
            if (sender == btnReset_R_ADD || sender == btnResetR || sender == btnReset) tbAddR.Value = 0;
            if (sender == btnReset_G_ADD || sender == btnResetG || sender == btnReset) tbAddG.Value = 0;
            if (sender == btnReset_B_ADD || sender == btnResetB || sender == btnReset) tbAddB.Value = 0;
            if (sender == btnReset_A_SCL || sender == btnResetA || sender == btnReset) tbScaleA.Value = (int)(TRACKBARMAX / ChannelOptions.MAXSCALE); //scale = 1.0 / MAXSCALE * TRACKBARMAX; 1.0 is default
            if (sender == btnReset_R_SCL || sender == btnResetR || sender == btnReset) tbScaleR.Value = (int)(TRACKBARMAX / ChannelOptions.MAXSCALE); //scale
            if (sender == btnReset_G_SCL || sender == btnResetG || sender == btnReset) tbScaleG.Value = (int)(TRACKBARMAX / ChannelOptions.MAXSCALE); //scale
            if (sender == btnReset_B_SCL || sender == btnResetB || sender == btnReset) tbScaleB.Value = (int)(TRACKBARMAX / ChannelOptions.MAXSCALE); //scale

            if (sender == btnResetA || sender == btnReset)
            {
                cbSrcA.SelectedItem = SourceEnum.Default;
                xbEnA.Checked = xbEnAddA.Checked = xbEnScaleA.Checked = xbInvA.Checked = false;

            }

            if (sender == btnResetR || sender == btnReset)
            {
                cbSrcR.SelectedItem = SourceEnum.Default;
                xbEnR.Checked = xbEnAddR.Checked = xbEnScaleR.Checked = xbInvR.Checked = false;
            }

            if (sender == btnResetG || sender == btnReset)
            {
                cbSrcG.SelectedItem = SourceEnum.Default;
                xbEnG.Checked = xbEnAddG.Checked = xbEnScaleG.Checked = xbInvG.Checked = false;
            }

            if (sender == btnResetB || sender == btnReset)
            {
                cbSrcB.SelectedItem = SourceEnum.Default;
                xbEnB.Checked = xbEnAddB.Checked = xbEnScaleB.Checked = xbInvB.Checked = false;
            }

            if (sender == btnReset)
            {
                updatingOrder = true;
                cbFirst.SelectedIndex = (int)ChannelsEnum.Red;
                cbSecond.SelectedIndex = (int)ChannelsEnum.Green;
                cbThird.SelectedIndex = (int)ChannelsEnum.Blue;
                cbForth.SelectedIndex = (int)ChannelsEnum.Alpha;
                updatingOrder = false;
            }

            FinishTokenUpdate();
        }

        bool updatingOrder = false;
        private void ChannelOrder_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (updatingOrder) return; //prevent stack overflow

            updatingOrder = true;

            int channel = ((ComboBox)sender).SelectedIndex;
            int inxDuplicate = -1;
            int cbInx = 0;

            if(sender == cbFirst) cbInx = 0;
            if(sender == cbSecond) cbInx = 1;
            if(sender == cbThird) cbInx = 2;
            if(sender == cbForth) cbInx = 3;

            //The following finds the channel that is no longer set and sets it to
            //the duplicate

            bool []channelFound = new bool[] { false, false, false, false };
            for (int n = 0; n < 4; n++)
            {
                if (processOrder[n].SelectedIndex == -1) break;

                channelFound[processOrder[n].SelectedIndex] = true;

                //find duplicate
                if (processOrder[n].SelectedIndex == channel && n != cbInx)
                {
                    inxDuplicate = n;
                }
            }
            if (inxDuplicate != -1)
            {
                for (int n = 0; n < 4; n++)
                {
                    if (channelFound[n] == false) processOrder[inxDuplicate].SelectedIndex = n;
                }
            }
             
            updatingOrder = false;

            FinishTokenUpdate();
        }

        //label control clicked
        private void ShiftChannelRight_Click(object sender, EventArgs e)
        {
            int temp = this.processOrder[3].SelectedIndex;

            updatingOrder = true;
            processOrder[3].SelectedIndex = processOrder[2].SelectedIndex;
            processOrder[2].SelectedIndex = processOrder[1].SelectedIndex;
            processOrder[1].SelectedIndex = processOrder[0].SelectedIndex;
            processOrder[0].SelectedIndex = temp;
            updatingOrder = false;

            FinishTokenUpdate();

        }

    }
}
