﻿using System;


namespace Matthew.ChannelMixEffect
{
    //sources are the source value to set the channel
    public enum SourceEnum
    {
        Alpha, Red, Green, Blue, Default, Set, Intensity
    }
    public enum ChannelsEnum
    {
        Alpha, Red, Green, Blue
    }

    //options for channel
    public class ChannelOptions
    {
        public static double MAXSCALE = 10.0;

        public bool enable;
        public SourceEnum source;
        public int set;                 //value to set channel
        public int add;                 //-255 .. 255,
        public double scale;            //0 - n
        public bool invert;
        public bool addEn;
        public bool scaleEn;
        public ChannelsEnum channel;

        public ChannelOptions(ChannelsEnum channel)
        {
            enable = false;
            source = SourceEnum.Default;
            set = 0;
            add = 0;
            scale = 1.0;
            invert = false;
            addEn = false;
            scaleEn = false;
            this.channel = channel;
        }

        public ChannelOptions(ChannelOptions other)
        {
            enable = other.enable;
            source = other.source;
            set = other.set;
            add = other.add;
            scale = other.scale;
            invert = other.invert;
            addEn = other.addEn;
            scaleEn = other.scaleEn;
            channel = other.channel;
        }
    }

    public class ChannelMixToken : PaintDotNet.Effects.EffectConfigToken
    {
        public ChannelOptions A, R, G, B;
        public ChannelOptions[] processOrder;

        public ChannelMixToken()
            : base()
        {
            A = new ChannelOptions(ChannelsEnum.Alpha);
            R = new ChannelOptions(ChannelsEnum.Red);
            G = new ChannelOptions(ChannelsEnum.Green);
            B = new ChannelOptions(ChannelsEnum.Blue);

            processOrder = new ChannelOptions[] { R, G, B, A };
        }

        //Copy Constructor
        public ChannelMixToken(ChannelMixToken other)
            : base(other)
        {
            A = new ChannelOptions(other.A);
            R = new ChannelOptions(other.R);
            G = new ChannelOptions(other.G);
            B = new ChannelOptions(other.B);

            processOrder = new ChannelOptions[4];
            for (int n = 0; n < 4; n++)
            {
                switch (other.processOrder[n].channel)
                {
                    case ChannelsEnum.Alpha: processOrder[n] = A; break;
                    case ChannelsEnum.Red: processOrder[n] = R; break;
                    case ChannelsEnum.Green: processOrder[n] = G; break;
                    case ChannelsEnum.Blue: processOrder[n] = B; break;
                }
            }
        }


        public override object Clone()
        {
            return new ChannelMixToken(this);
        }


        public void SetChannelOrder(ChannelsEnum channel, int pos)
        {
            ChannelOptions temp = processOrder[pos];

            for (int n = 0; n < 4; n++) {
                if (processOrder[n].channel == channel) 
                {
                    processOrder[pos] = processOrder[n];
                    processOrder[n] = temp;
                }
            }
        }

        public void ShrChannelOrder()
        {
            ChannelOptions temp = processOrder[3];

            processOrder[3] = processOrder[2];
            processOrder[2] = processOrder[1];
            processOrder[1] = processOrder[0];
            processOrder[0] = temp;
        }

    }
}
