﻿using System;
using System.Text;
using System.Reflection;
using PaintDotNet;
using PaintDotNet.Effects;
using PaintDotNet.IndirectUI;
using PaintDotNet.PropertySystem;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
//using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;



namespace Matthew
{

    [PluginSupportInfo(typeof(PluginSupportInfo), DisplayName = "MattChannelMixer")]
    public class ChannelMixEffectOLD : PropertyBasedEffect
    {
        #region Objects and Types
        public enum PropertyNames
        {
            //Alpha
            Channel,
            Source,
            SetFlag,
            SetAmount,
            ScaleFlag,
            ScaleAmount,
            AddFlag,
            AddAmount,
            Invert,
        }

        enum SourceEnum
        {
            Alpha, Red, Green, Blue, Default, Set, Intensity
        }
        enum ChannelsEnum
        {
            Alpha, Red, Green, Blue
        }

        class ChannelBlendOptions
        {

            public SourceEnum source = SourceEnum.Default;
            public int set = 0;
            public int add = 0;
            public double mult = 1.0;
            public bool invert = false;
            public bool addFlag = false;
            public bool scaleFlag = false;
        }
        #endregion


        #region Static Variables
        public static string StaticName
        {
            get
            {
                return "Channel Mixer";
            }
        }

        public static Image StaticIcon
        {
            get
            {
                Bitmap bm = null;
                Assembly a = Assembly.GetExecutingAssembly();

                using (System.IO.Stream bmStream = a.GetManifestResourceStream("Matthew.Icon.png"))
                {
                    bm = new Bitmap(bmStream);
                }
                return bm;
            }
        }
        #endregion

        ChannelBlendOptions []channels;
        ChannelBlendOptions cbA, cbR, cbG, cbB;

        object[] channelNames;
        object[] sourceNames;


        public ChannelMixEffectOLD()
            : base(StaticName, StaticIcon, SubmenuNames.Render, EffectFlags.Configurable)
        {
            
            cbA = new ChannelBlendOptions();
            cbR = new ChannelBlendOptions();
            cbG = new ChannelBlendOptions();
            cbB = new ChannelBlendOptions();
            channels = new ChannelBlendOptions[] { cbA,cbB,cbG,cbB };
            channelNames = new object[] { ChannelsEnum.Alpha, ChannelsEnum.Red, ChannelsEnum.Green, ChannelsEnum.Blue };
            sourceNames = new object[] { SourceEnum.Default,SourceEnum.Set,
                                       SourceEnum.Alpha, SourceEnum.Red, SourceEnum.Green, SourceEnum.Blue,
                                       SourceEnum.Intensity};
        }


        protected override PropertyCollection OnCreatePropertyCollection()
        {
            //System.Windows.Forms.MessageBox.Show("OnCreatePropertyCollection");

            List<Property> props = new List<Property>();

            StaticListChoiceProperty chanProp = new StaticListChoiceProperty(PropertyNames.Channel, channelNames, 0);
            props.Add(chanProp);
            props.Add(new StaticListChoiceProperty(PropertyNames.Source, sourceNames, 0));     
            props.Add(new Int32Property(PropertyNames.SetAmount, 0, 0, 255));
            props.Add(new BooleanProperty(PropertyNames.AddFlag, false));
            props.Add(new Int32Property(PropertyNames.AddAmount, 0, -255, 255)); 
            props.Add(new BooleanProperty(PropertyNames.ScaleFlag, false));
            props.Add(new DoubleProperty(PropertyNames.ScaleAmount, 1.0, 0, 20));
            props.Add(new BooleanProperty(PropertyNames.Invert, false));

            return new PropertyCollection(props);
        }

        ChannelsEnum currChannel = ChannelsEnum.Alpha;

        protected override ControlInfo OnCreateConfigUI(PropertyCollection props)
        {
            //System.Windows.Forms.MessageBox.Show("OnCreateConfigUI");

            ControlInfo configUI = CreateDefaultConfigUI(props);

            configUI.SetPropertyControlValue(PropertyNames.SetAmount, ControlInfoPropertyNames.SliderLargeChange, 5);
            configUI.SetPropertyControlValue(PropertyNames.AddAmount, ControlInfoPropertyNames.SliderSmallChange, 5);
            configUI.SetPropertyControlValue(PropertyNames.ScaleAmount, ControlInfoPropertyNames.SliderLargeChange, 1.0);
            configUI.SetPropertyControlValue(PropertyNames.ScaleAmount, ControlInfoPropertyNames.SliderSmallChange, 0.1);

            configUI.SetPropertyControlValue(PropertyNames.Source, ControlInfoPropertyNames.DisplayName, "SOURCE");
            configUI.SetPropertyControlValue(PropertyNames.Source, ControlInfoPropertyNames.Description, "");

            configUI.SetPropertyControlValue(PropertyNames.SetAmount, ControlInfoPropertyNames.DisplayName, "ALPHA BASE");
            configUI.SetPropertyControlValue(PropertyNames.SetAmount, ControlInfoPropertyNames.Description, "Amount to set alpha (a = x)");

            configUI.SetPropertyControlValue(PropertyNames.AddFlag, ControlInfoPropertyNames.DisplayName, "ALPHA ADDITION");
            configUI.SetPropertyControlValue(PropertyNames.AddFlag, ControlInfoPropertyNames.Description, "Enabled");
            configUI.SetPropertyControlValue(PropertyNames.AddAmount, ControlInfoPropertyNames.DisplayName, "");
            configUI.SetPropertyControlValue(PropertyNames.AddAmount, ControlInfoPropertyNames.Description, "Amount to add to alpha (a = a + x)");

            configUI.SetPropertyControlValue(PropertyNames.ScaleFlag, ControlInfoPropertyNames.DisplayName, "ALPHA SCALING");
            configUI.SetPropertyControlValue(PropertyNames.ScaleFlag, ControlInfoPropertyNames.Description, "Enabled");
            configUI.SetPropertyControlValue(PropertyNames.ScaleAmount, ControlInfoPropertyNames.DisplayName, "");
            configUI.SetPropertyControlValue(PropertyNames.ScaleAmount, ControlInfoPropertyNames.Description, "Amount to scale alpha (a = x*a)");

            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.DisplayName, "OTHER");
            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.Description, "Invert Alpha (a = 1 - a)");
            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.DisplayName, "Enable");

            return configUI;
        }


        protected override void OnSetRenderInfo(PropertyBasedEffectConfigToken newToken, RenderArgs dstArgs, RenderArgs srcArgs)
        {
            //System.Windows.Forms.MessageBox.Show("OnSetRenderInfo");

            ChannelBlendOptions cb;

            ChannelsEnum channel = (ChannelsEnum)newToken.GetProperty<StaticListChoiceProperty>(PropertyNames.Channel).Value;

            if (channel == currChannel)
            {            
                cb = channels[(int)currChannel];
                SaveChannelInfo(newToken.Properties, cb);
            }
            else
            {
                currChannel = channel;

                cb = channels[(int)currChannel];
                LoadChannelInfo(newToken, cb);
            }

            base.OnSetRenderInfo(newToken, dstArgs, srcArgs);
        }

        //save ChannelBlendOptions to prop
        void SaveChannelInfo(PropertyCollection props, ChannelBlendOptions cb)
        {
            props[PropertyNames.Source].Value = cb.source;
            props[PropertyNames.SetAmount].Value = cb.set;
            props[PropertyNames.AddAmount].Value = cb.add;
            props[PropertyNames.ScaleAmount].Value = cb.mult;
            props[PropertyNames.AddFlag].Value = cb.addFlag;
            props[PropertyNames.ScaleFlag].Value = cb.scaleFlag;
            props[PropertyNames.Invert].Value = cb.invert;
        }
        void LoadChannelInfo(PropertyBasedEffectConfigToken newToken, ChannelBlendOptions cb)
        {
            cb.source = (SourceEnum)newToken.GetProperty<StaticListChoiceProperty>(PropertyNames.Source).Value;
            cb.invert = newToken.GetProperty<BooleanProperty>(PropertyNames.Invert).Value;
            cb.addFlag = newToken.GetProperty<BooleanProperty>(PropertyNames.AddFlag).Value;
            cb.scaleFlag = newToken.GetProperty<BooleanProperty>(PropertyNames.ScaleFlag).Value;
            cb.set = newToken.GetProperty<Int32Property>(PropertyNames.SetAmount).Value;
            cb.add = newToken.GetProperty<Int32Property>(PropertyNames.AddAmount).Value;
            cb.mult = newToken.GetProperty<DoubleProperty>(PropertyNames.ScaleAmount).Value;
        }

        protected override void OnCustomizeConfigUIWindowProperties(PropertyCollection props)
        {
            //System.Windows.Forms.MessageBox.Show("OnCustomizeConfigUIWindowProperties");


            // Change the effect's window title
            props[ControlInfoPropertyNames.WindowTitle].Value = "Matts Channel Mixer v1.0";

            /*
            if (props[PropertyNames.Channel] != null)
            {

                ChannelsEnum dest = (ChannelsEnum)props[PropertyNames.Channel].Value;
                cb = channels[(int)dest];

                props[PropertyNames.Source].Value = cb.source;
                props[PropertyNames.SetAmount].Value = cb.set;
                props[PropertyNames.AddAmount].Value = cb.add;
                props[PropertyNames.ScaleAmount].Value = cb.mult;
                props[PropertyNames.AddFlag].Value = cb.addFlag;
                props[PropertyNames.ScaleFlag].Value = cb.scaleFlag;
                props[PropertyNames.Invert].Value = cb.invert;
            }
             */

            base.OnCustomizeConfigUIWindowProperties(props); 
        }


        protected override void OnRender(Rectangle[] rois, int startIndex, int length)
        {
            if (length == 0) return;
            for (int i = startIndex; i < startIndex + length; ++i)
            {
                Render(DstArgs.Surface, SrcArgs.Surface, rois[i]);
            }
        }

        void Render(Surface dst, Surface src, Rectangle rc)
        {          
            for (int y = rc.Top; y < rc.Bottom; ++y)
            {
                for (int x = rc.Left; x < rc.Right; ++x)
                {          
                    ColorBgra color = src[x, y];
                    int intensity = color.GetIntensityByte();

                    //ALPHA      
                    double value = cbA.source == SourceEnum.Set ? cbA.set :
                        cbA.source == SourceEnum.Intensity ? intensity :
                        cbA.source == SourceEnum.Red ? color.R :
                        cbA.source == SourceEnum.Green ? color.G :
                        cbA.source == SourceEnum.Blue ? color.B :
                        color.A;
                    if (cbA.addFlag) value += cbA.add;
                    if (cbA.scaleFlag) value *= cbA.mult;
                    //CLAMP 0 - 255
                    color.A = (byte)(
                        value < 0 ? 0 :
                        value > 255 ? 255 :
                        value);
                    if (cbA.invert) color.A = (byte)(255 - color.A);

                    //RED
                    value = cbR.source == SourceEnum.Set ? cbR.set :
                        cbR.source == SourceEnum.Intensity ? intensity :
                        cbR.source == SourceEnum.Red ? color.R :
                        cbR.source == SourceEnum.Green ? color.G :
                        cbR.source == SourceEnum.Blue ? color.B :
                        color.A;
                    if (cbR.addFlag) value += cbR.add;
                    if (cbR.scaleFlag) value *= cbR.mult;
                    //CLAMP 0 - 255
                    color.A = (byte)(
                        value < 0 ? 0 :
                        value > 255 ? 255 :
                        value);
                    if (cbR.invert) color.R = (byte)(255 - color.R);

                    //GREEN
                    value = cbG.source == SourceEnum.Set ? cbG.set :
                        cbG.source == SourceEnum.Intensity ? intensity :
                        cbG.source == SourceEnum.Red ? color.R :
                        cbG.source == SourceEnum.Green ? color.G :
                        cbG.source == SourceEnum.Blue ? color.B :
                        color.A;
                    if (cbG.addFlag) value += cbG.add;
                    if (cbG.scaleFlag) value *= cbG.mult;
                    //CLAMP 0 - 255
                    color.G = (byte)(
                        value < 0 ? 0 :
                        value > 255 ? 255 :
                        value);
                    if (cbG.invert) color.G = (byte)(255 - color.G);

                    //BLUE
                    value = cbB.source == SourceEnum.Set ? cbB.set :
                        cbB.source == SourceEnum.Intensity ? intensity :
                        cbB.source == SourceEnum.Red ? color.R :
                        cbB.source == SourceEnum.Green ? color.G :
                        cbB.source == SourceEnum.Blue ? color.B :
                        color.A;
                    if (cbB.addFlag) value += cbB.add;
                    if (cbB.scaleFlag) value *= cbB.mult;
                    //CLAMP 0 - 255
                    color.B = (byte)(
                        value < 0 ? 0 :
                        value > 255 ? 255 :
                        value);
                    if (cbB.invert) color.B = (byte)(255 - color.B);

                    dst[x,y] = color;
                }
            }
        }
    }
}